PROGRAM Compute_dlnPkdz
  ! A program for computing dlnPk/dz using:
  ! 1) Peacock and Dodds [Peacock and Dodds, MNRAS, 280, L19 (1996)]
  ! 2) Ma et al. [Ma et al., ApJL, 521, L1 (1999)]
  ! 3) Halofit [Smith et al., MNRAS, 341, 1311 (2003)]
  ! - k is in units of h Mpc^-1
  ! - P(k) is in units of h^-3 Mpc^3
  ! January 1, 2012: E.Komatsu
  IMPLICIT none
  character(len=128) :: filename(3)
  integer :: nz=901 ! # of data points in redshift in the input data file
  integer :: nk=691 ! # of data points in wavenumber in the input data file
  double precision :: zmax=9d0 ! maximum redshift in the input data file
  double precision, allocatable, dimension(:,:) :: pkz,y2a
  double precision, allocatable, dimension(:) :: redshift,k_ov_h
  double precision :: dlnPkdz,kout_ov_h,zout,dln1pz
  integer :: ik,iz,j
! ask for the wavenumber
  print*,'wavenumber [h Mpc^-1]?'
  read*,kout_ov_h
  if(kout_ov_h<1d-4)then
     print*,kout_ov_h,'is smaller than 1e-4 Mpc/h.'
     stop
  else if(kout_ov_h>1d2)then
     print*,kout_ov_h,'is greater than 1e2 Mpc/h.'
     stop
  endif
! read in and tabulate P(k,z)
  ALLOCATE(pkz(nz,nk),y2a(nz,nk),redshift(nz),k_ov_h(nk))
  do j=1,3
     filename(1)='redshift_wavenumber_pkpd96.txt'
     filename(2)='redshift_wavenumber_pkma99.txt'
     filename(3)='redshift_wavenumber_pkhalofit.txt'
     print*,'read in '//trim(filename(j)) &
          //' and generate an interpolation table'
     open(1,file=filename(j),status='old')
     do iz=1,nz
        do ik=1,nk
           read(1,*)redshift(iz),k_ov_h(ik),pkz(iz,ik)
        enddo
     enddo
     close(1)
     CALL splie2(redshift,dlog(k_ov_h),dlog(pkz),nz,nk,y2a)
     filename(1)='redshift_dlnpkdzpd96.txt'
     filename(2)='redshift_dlnpkdzma99.txt'
     filename(3)='redshift_dlnpkdzhalofit.txt'
     open(2,file=filename(j),status='unknown')
     dln1pz=dlog((1d0+zmax))/1d2 ! dlog(1+zmax)/(# of points in redshift)
     do iz=1,100
        zout=dexp(dble(iz)*dln1pz)-1d0
        CALL splin2_der1(redshift,dlog(k_ov_h),dlog(pkz),y2a,nz,nk, &
             zout,dlog(kout_ov_h),dlnPkdz)
        write(2,'(2E18.8)')zout,dlnPkdz
     enddo
     close(2)
  enddo
  DEALLOCATE(pkz,y2a,redshift,k_ov_h)
END PROGRAM Compute_dlnPkdz
